# Generated by Django 3.2.5 on 2023-04-18 22:56

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Flow',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('flow_name', models.CharField(help_text='必填', max_length=255, verbose_name='流程名称')),
                ('module_name', models.CharField(blank=True, choices=[('USER', '用户(USER)'), ('THREAD', '信息(THREAD)'), ('COMMENT', '评论(COMMENT)'), ('FINANCE', '资金(FINANCE)'), ('ENROLL', '报名(ENROLL)'), ('RESOURCE', '资源(RESOURCE)'), ('DICTIONARY', '字典(DICTIONARY)'), ('PAYMENT', '支付(PAYMENT)'), ('ROLE', '角色(ROLE)')], max_length=32, null=True, verbose_name='模块名称')),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='流程描述')),
            ],
            options={
                'verbose_name': '1.流程主表',
                'verbose_name_plural': '1.流程主表',
                'db_table': 'flow_flow',
            },
        ),
        migrations.CreateModel(
            name='FlowAction',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.CharField(db_index=True, help_text='必填', max_length=255, unique=True, verbose_name='动作关键字')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='动作名称')),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='动作描述')),
                ('config', models.JSONField(blank=True, null=True, verbose_name='前端配置')),
            ],
            options={
                'verbose_name': '3.流程动作表',
                'verbose_name_plural': '3.流程动作表',
                'db_table': 'flow_action',
            },
        ),
        migrations.CreateModel(
            name='FlowNode',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('node_name', models.CharField(blank=True, help_text='节点名称建议使用下一流程状态命名。例如：已付款的下一状态是接单，则写待接单', max_length=255, null=True, verbose_name='节点名称')),
                ('node_value', models.CharField(blank=True, default='', max_length=255, null=True, verbose_name='节点值')),
                ('module_name', models.CharField(blank=True, choices=[('USER', '用户(USER)'), ('THREAD', '信息(THREAD)'), ('COMMENT', '评论(COMMENT)'), ('FINANCE', '资金(FINANCE)'), ('ENROLL', '报名(ENROLL)'), ('RESOURCE', '资源(RESOURCE)'), ('DICTIONARY', '字典(DICTIONARY)'), ('PAYMENT', '支付(PAYMENT)'), ('ROLE', '角色(ROLE)')], max_length=32, null=True, verbose_name='模块名称')),
                ('flow_number', models.IntegerField(blank=True, null=True, verbose_name='流程号')),
                ('status_code', models.IntegerField(blank=True, db_index=True, help_text='订单状态表示法：0完成、1 留空或非、2下单、3接单、4付款、5发货、6收货、7退货、8评价、9冗余', null=True, verbose_name='状态码')),
                ('summary', models.CharField(blank=True, max_length=1024, null=True, verbose_name='摘要')),
                ('description', models.CharField(blank=True, max_length=1024, null=True, verbose_name='描述')),
                ('config', models.JSONField(blank=True, default={}, help_text='前端状态配置', null=True, verbose_name='节点配置')),
                ('flow_id', models.ForeignKey(db_column='flow_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flow', verbose_name='流程ID')),
            ],
            options={
                'verbose_name': '2.流程节点表',
                'verbose_name_plural': '2.流程节点表',
                'db_table': 'flow_node',
            },
        ),
        migrations.CreateModel(
            name='FlowToCategory',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('category_id', models.IntegerField(blank=True, null=True, verbose_name='类别ID')),
                ('flow', models.ForeignKey(db_column='flow_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flow', verbose_name='流程ID')),
            ],
            options={
                'verbose_name': '2.流程节点类别表',
                'verbose_name_plural': '2.流程节点类别表',
                'db_table': 'flow_to_category',
            },
        ),
        migrations.CreateModel(
            name='FlowRecord',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.BigIntegerField(db_index=True, verbose_name='用户ID')),
                ('belong_role_id', models.IntegerField(blank=True, help_text='操作该记录的所属角色(即操作人)', null=True, verbose_name='所属角色ID')),
                ('summary', models.CharField(blank=True, db_index=True, help_text='摘要自动生成。例如：“【张三】检查完成【马云】的“XXX”资料，并信息补充”', max_length=255, null=True, verbose_name='摘要(自动)')),
                ('flow_node_id', models.ForeignKey(db_column='flow_node_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flownode', verbose_name='流程节点ID')),
            ],
            options={
                'verbose_name_plural': '7. 流程记录表',
                'db_table': 'flow_record',
            },
        ),
        migrations.CreateModel(
            name='FlowNodeToAction',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, default='', max_length=255, null=True, verbose_name='描述')),
                ('ttl', models.IntegerField(blank=True, default=-1, null=True, verbose_name='生命周期')),
                ('role_list', models.JSONField(blank=True, null=True, verbose_name='角色列表')),
                ('user_list', models.JSONField(blank=True, null=True, verbose_name='用户列表')),
                ('is_automatic', models.BooleanField(blank=True, default=0, null=True, verbose_name='是否自动触发')),
                ('flow_action_id', models.ForeignKey(db_column='flow_action_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flowaction', verbose_name='流程动作ID')),
                ('flow_node_id', models.ForeignKey(db_column='flow_node_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flownode', verbose_name='流程节点ID')),
                ('flow_to_fail_node_id', models.ForeignKey(blank=True, db_column='flow_to_fail_node_id', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='xj_flow.flownode', verbose_name='流向失败的节点ID')),
                ('flow_to_node_id', models.ForeignKey(blank=True, db_column='flow_to_node_id', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='xj_flow.flownode', verbose_name='流向节点ID')),
            ],
            options={
                'verbose_name': '4.流程节点多对多动作表',
                'verbose_name_plural': '4.流程节点多对多动作表',
                'db_table': 'flow_node_to_action',
                'unique_together': {('flow_node_id', 'flow_action_id', 'flow_to_node_id')},
            },
        ),
        migrations.CreateModel(
            name='FlowNodeLog',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('user_id', models.BigIntegerField(db_index=True, verbose_name='用户ID')),
                ('belong_role_id', models.IntegerField(blank=True, help_text='操作该日志的所属角色(即操作人)', null=True, verbose_name='所属角色ID')),
                ('summary', models.CharField(blank=True, db_index=True, help_text='摘要自动生成。例如：“【张三】检查完成【马云】的“XXX”资料，并信息补充”', max_length=255, null=True, verbose_name='摘要(自动)')),
                ('create_time', models.DateTimeField(default=django.utils.timezone.now, verbose_name='创建时间')),
                ('flow_node_id', models.ForeignKey(db_column='flow_node_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flownode', verbose_name='流程节点ID')),
                ('flow_record_id', models.ForeignKey(db_column='flow_record_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flowrecord', verbose_name='流程节点ID')),
            ],
            options={
                'verbose_name_plural': '8. 流程节点日志表',
                'db_table': 'flow_node_log',
            },
        ),
        migrations.CreateModel(
            name='FlowNodeActionRule',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('rule_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='规则名称')),
                ('rule_sort', models.IntegerField(blank=True, db_index=True, null=True, verbose_name='规则顺序')),
                ('inflow_service', models.CharField(blank=True, choices=[('enroll_detail', '报名详细(enroll_detail)'), ('thread_detail', '信息详细(thread_detail)'), ('payment_detail', '支付详细(payment_detail)')], db_index=True, max_length=32, null=True, verbose_name='流入服务')),
                ('inflow_module', models.CharField(blank=True, choices=[('USER', '用户(USER)'), ('THREAD', '信息(THREAD)'), ('COMMENT', '评论(COMMENT)'), ('FINANCE', '资金(FINANCE)'), ('ENROLL', '报名(ENROLL)'), ('RESOURCE', '资源(RESOURCE)'), ('DICTIONARY', '字典(DICTIONARY)'), ('PAYMENT', '支付(PAYMENT)'), ('ROLE', '角色(ROLE)')], db_index=True, max_length=32, null=True, verbose_name='流入模块')),
                ('inflow_field', models.CharField(blank=True, max_length=32, null=True, verbose_name='流入字段')),
                ('outflow_module', models.CharField(blank=True, choices=[('USER', '用户(USER)'), ('THREAD', '信息(THREAD)'), ('COMMENT', '评论(COMMENT)'), ('FINANCE', '资金(FINANCE)'), ('ENROLL', '报名(ENROLL)'), ('RESOURCE', '资源(RESOURCE)'), ('DICTIONARY', '字典(DICTIONARY)'), ('PAYMENT', '支付(PAYMENT)'), ('ROLE', '角色(ROLE)')], db_index=True, help_text='默认与流入模块相同', max_length=32, null=True, verbose_name='流出模块')),
                ('outflow_field', models.CharField(blank=True, help_text='默认与流入字段相同。如果流出字段不存在则自动创建', max_length=32, null=True, verbose_name='流出字段')),
                ('default_value', models.CharField(blank=True, max_length=255, null=True, verbose_name='默认值')),
                ('expression_string', models.CharField(blank=True, max_length=2048, null=True, verbose_name='逻辑表达式')),
                ('python_script', models.TextField(blank=True, null=True, verbose_name='帕森脚本')),
                ('flow_node_to_action_id', models.ForeignKey(db_column='flow_node_to_action_id', db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flownodetoaction', verbose_name='流程节点动作ID')),
            ],
            options={
                'verbose_name_plural': '6. 流程节点动作规则表',
                'db_table': 'flow_node_action_rule',
            },
        ),
        migrations.AddField(
            model_name='flownode',
            name='many_flow_action_id',
            field=models.ManyToManyField(through='xj_flow.FlowNodeToAction', to='xj_flow.FlowAction', verbose_name='多对多流程动作ID'),
        ),
        migrations.CreateModel(
            name='FlowLastStatus',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('module_code', models.CharField(max_length=255, verbose_name='模型编码')),
                ('table_code', models.CharField(max_length=255, verbose_name='表编码')),
                ('relate_data_id', models.IntegerField(verbose_name='执行的主键ID')),
                ('user_id', models.IntegerField(blank=True, db_index=True, null=True, verbose_name='用户ID')),
                ('flow_node_id', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flownode', verbose_name='当前用户的节点')),
            ],
            options={
                'verbose_name_plural': '9. 流程节点日志表',
                'db_table': 'flow_last_status',
            },
        ),
        migrations.CreateModel(
            name='FlowActionToOperator',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('role_id', models.IntegerField(blank=True, help_text='操作该记录的所属角色(即操作组)', null=True, verbose_name='操作角色ID')),
                ('user_id', models.BigIntegerField(blank=True, help_text='操作该记录的所属用户(即操作人)', null=True, verbose_name='操作人员ID')),
                ('flow_action_id', models.ForeignKey(db_column='flow_action_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_flow.flowaction', verbose_name='流程动作ID')),
            ],
            options={
                'verbose_name': '5.流程动作多对多操作人表',
                'verbose_name_plural': '5.流程动作多对多操作人表',
                'db_table': 'flow_action_to_operator',
            },
        ),
    ]



# _*_coding:utf-8_*_

from rest_framework.views import APIView

from ..services.flow_basic_service import FlowBasicService
from ..utils.custom_response import util_response
from ..utils.custom_tool import format_params_handle, force_transform_type
from ..utils.request_params_wrapper import request_params_wrapper


class FlowNodeToActionList(APIView):
    @request_params_wrapper
    def get(self, *args, request_params=None, **kwargs):
        """
        流程作业
        """
        request_params, is_pass = force_transform_type(variable=request_params, var_type="dict", default={})
        request_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["flow_id", "need_custom_serialize", "flow_node_id", "flow_action_id", "flow_node_value", "flow_action", "flow_status_code_list", "is_using"],
            split_list=["flow_status_code_list"]
        )
        need_custom_serialize, is_pass = force_transform_type(
            variable=request_params.pop("need_custom_serialize", False),
            var_type="bool",
            default=False
        )
        flow_list, error_text = FlowBasicService.get_flow_node_to_action_list(
            params=request_params,
            need_custom_serialize=need_custom_serialize
        )
        return util_response(data=flow_list)



# encoding: utf-8
"""
@project: djangoModel->1
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 流程控制系统
@created_time: 2023/5/17 13:29
"""

from django.db import transaction
from django.db.models import F

from ..models import FlowNodeActionRule, FlowNode, FlowAction, FlowNodeToAction, FlowRecord
from ..utils.custom_tool import *
from ..utils.j_valuation import JExpression


class FlowProcessService:
    # 执行记录
    process_context = []
    current_context = {}
    # 流程基础信息
    flow_node_dict = {}
    action_info_dict = {}
    flow_action_node_map = {}
    flow_constant_quantity_map = {}
    # 用户信息
    user_info = {}

    def push_run_action(self, func, result):
        """
        记录方法的执行过程
        :param result: 执行结果
        :param func: 方法指针
        :return: None
        """
        # 记录执行结果
        if not self.current_context.get("rule_action_result") or not isinstance(self.current_context["rule_action_result"], dict):
            self.current_context["rule_action_result"] = {}
        func_name = getattr(func, "__name__", str(func))
        self.current_context["rule_action_result"][func_name] = result

    def flow_constant_quantity(self, source_params=None, is_update=False):
        """
        获取流程常量映射
        :param source_params:用户自定义传输
        :param is_update: 是否在原有基础上再次更新
        :return: self.flow_constant_quantity_map
        """
        if source_params is None:
            source_params = {}
        if not self.flow_constant_quantity_map or is_update:
            self.flow_constant_quantity_map.update(self.action_info_dict)
            self.flow_constant_quantity_map.update(self.flow_node_dict)
            self.flow_constant_quantity_map.update(self.flow_action_node_map)
            if source_params and isinstance(source_params, dict):
                self.flow_constant_quantity_map.update(source_params)
        return self.flow_constant_quantity_map

    def __do_by_default_value(self, source_params, rule):
        """通过默认值进行赋值"""
        outflow_field = rule.get("outflow_field")
        default_value = rule.get("default_value")
        if not outflow_field or not default_value:
            return source_params
        # 如果是变量表达式则进行替换
        flow_constant_quantity = self.flow_constant_quantity()
        default_value, parsed_variable_map = JExpression.parse_variables(default_value, flow_constant_quantity, default_value="")
        # 如果是json字符串则json格式化
        default_value = parse_json(default_value)
        source_params.setdefault(outflow_field, default_value)
        # 进行记录流程
        self.push_run_action("set_default", {"field": outflow_field, "value": default_value})
        return source_params

    def __do_by_expression(self, source_params, rule):
        """通过表达式进行赋值"""
        outflow_field = rule.get("outflow_field", None)
        setting_expression_string = rule.get("expression_string", None)
        if outflow_field and setting_expression_string:
            # 公式字符串，变量解析替换
            expression_string, parsed_variable_map = JExpression.parse_variables(
                setting_expression_string,
                source_params
            )
            # 实例化计算器类，并且计算结果
            calculator = JExpression()
            data, err = calculator.process(expression_string)
            # 结果赋值
            data = round(data, 2) if isinstance(data, float) else data
            source_params[rule["outflow_field"]] = data
            # 收录执行日志
            self.push_run_action("parse_expression", {"field": outflow_field, "value": data})

        return source_params

    def __do_by_python_script(self, source_params, rule):
        """执行脚本方法"""
        inflow_field = rule.get("outflow_field", None)
        python_script = rule.get("python_script", None)
        if inflow_field and python_script:
            # 创建栈空间，并传入流入流程的字段和值
            params = {inflow_field: source_params[inflow_field]}
            # TODO 这个python_script 最好做成密文存取
            exec(python_script, params)
            inflow_filed_result = params.get(inflow_field, None)
            # 处理后的结果赋值
            source_params[inflow_field] = inflow_filed_result if inflow_filed_result else source_params[inflow_field]
            self.push_run_action("run_python_script", {"field": inflow_field, "value": source_params[inflow_field]})
        return source_params

    def do_once_flow_in_service(
            self,
            flow_node_id=None,
            flow_action_id=None,
            flow_node_value=None,
            flow_action_value=None,
            source_params: dict = None,
            user_info: dict = None,
            **kwargs
    ):
        """
        根据流程节点调用配置好的服务方法，也可以执行表达式，脚本，默认字。
        执行一个节点对动作的使用中的规则。
        @param flow_node_id 流程节点ID
        @param flow_action_id 希望处理的流程动作ID
        @param source_params 需要处理的原数据
        :param user_info: 用户信息
        :param flow_action_value: 动作Value
        :param flow_node_value:
        :return flow_id, msg
        """
        # ================ section 参数初始化  start ==========================
        self.source_params, is_pass = force_transform_type(variable=source_params, var_type="dict", default={})
        self.source_params = self.source_params.copy()
        self.user_info, is_pass = force_transform_type(variable=user_info, var_type="dict", default={})
        flow_node_id, is_pass = force_transform_type(variable=flow_node_id, var_type="int")
        flow_action_id, is_pass = force_transform_type(variable=flow_action_id, var_type="int")
        if flow_node_id is None and flow_node_value is None and flow_action_id is None and flow_action_value is None:
            return None, None
        # ================ section 参数初始化  end   ==========================

        # ================ section 获取节点或者动作信息，减少链表  start ==========================
        # note 获取流程信息
        flow_node_obj = FlowNode.objects.annotate(node_config=F("config"), node_id=F("id")).values(
            "node_id", "flow_id", "node_name", "node_value", "module_name", "flow_number", "status_code", "node_config"
        )
        if flow_action_id:
            self.flow_node_dict = flow_node_obj.filter(node_id=flow_node_id).first()
        else:
            self.flow_node_dict = flow_node_obj.filter(node_value=flow_node_value).first()
        if not self.flow_node_dict:
            return None, "不是有效的流程节点参数"
        flow_node_id = self.flow_node_dict.get("node_id")
        # note 获取动作信息
        flow_action_obj = FlowAction.objects.annotate(action_config=F("config"), action_id=F("id")).values(
            "action_id", "action", "name", "description", "action_config",
        )
        if flow_action_id:
            self.action_info_dict = flow_action_obj.filter(action_id=flow_action_id).first()
        else:
            self.action_info_dict = flow_action_obj.filter(action=flow_action_value).first()
        if not self.action_info_dict:
            return None, "不是有效的流程动作参数"
        flow_action_id = self.action_info_dict.get("action_id")
        # ================ section 获取节点或者动作信息，减少连表查询 end  ==========================

        # ================ section 获取流程执行规则 start ==========================
        # note 获取流程映射
        self.flow_action_node_map = FlowNodeToAction.objects.annotate(
            next_node_value=F("flow_to_node__node_value"),
            next_node_code=F("flow_to_node__status_code"),
            flow_to_node_value=F("flow_to_node__node_value"),
            flow_to_node_code=F("flow_to_node__status_code")
        ).filter(
            flow_node_id=flow_node_id, flow_action_id=flow_action_id, is_using=1
        ).values().first()
        if not self.flow_action_node_map:
            return None, "没有给流程映射"
        flow_node_to_action_id = self.flow_action_node_map.get("id")
        flow_to_node_info = {
            "flow_to_node_id": self.flow_action_node_map.get("flow_to_node_id"),
            "flow_to_node_value": self.flow_action_node_map.get("flow_to_node_value"),
            "flow_to_node_code": self.flow_action_node_map.get("flow_to_node_code")
        }
        # note 获取流程规则
        flow_rule_obj = FlowNodeActionRule.objects.order_by("rule_sort").filter(
            is_using=1, flow_node_to_action_id=flow_node_to_action_id
        ).values(
            "id", "flow_node_to_action_id", "rule_name", "rule_sort",
            "inflow_service", "inflow_module", "inflow_field", "outflow_module", "outflow_field",
            "default_value", "expression_string", "python_script",
        )
        flow_rule_list = list(flow_rule_obj)
        if not flow_rule_list:
            return {"source_params": self.source_params}, "没有找到可执行流程"
        # ================ section 获取流程执行规则 end   ==========================

        # ================ section 执行流程规则 start ==========================
        sid = transaction.savepoint()  # 开始事务
        err_msg = None
        for item in flow_rule_list:
            self.current_context = {
                "rule_name": item["rule_name"],
                "rule_id": item["id"],
                "is_run_service": False
            }
            # note 参数初始化
            self.source_params = self.__do_by_default_value(self.source_params, item)
            self.source_params = self.__do_by_expression(self.source_params, item)
            self.source_params = self.__do_by_python_script(self.source_params, item)
            # note 开始执行相关服务
            if not item.get("inflow_module", None) or not item.get("inflow_service", None):
                self.process_context.append(self.current_context)
                continue
            # 配置了模块，但是没有对外开放，直接返回预处理的数据
            model = sys.modules.get(item["inflow_module"], None)
            if not model:
                self.process_context.append(self.current_context)
                continue
            # 加载需要的服务，如果没有可执行的服务方法，返回预处理的数据
            service = getattr(model, item["inflow_service"], None)
            if service:
                self.current_context["is_run_service"] = True
                try:
                    input_params = service_params_adapter(service, self.source_params)
                    data, err = service(**input_params)
                    self.push_run_action("run_service", {"service": item["inflow_service"], "result": data, "err": err})
                    self.process_context.append(self.current_context)
                    if err:
                        # 执行错误，则进行回滚，返回执行记录
                        err_msg = err
                        transaction.savepoint_rollback(sid)
                        break
                except Exception as e:
                    transaction.savepoint_rollback(sid)
                    err_msg = "停止运行，原因如下：" + str(e)
                    self.push_run_action("run_service", {"service": item["inflow_service"], "err": err_msg})
                    self.process_context.append(self.current_context)
                    break
        # ================ section 执行流程规则 end   ==========================
        # 完成执行，清除所有的点
        transaction.clean_savepoints()
        return {"source_params": self.source_params, "process_context": self.process_context, "flow_to_node_info": flow_to_node_info}, err_msg

    def save_record(self, result_dict: dict = None, user_info: dict = None):
        """
        保存流程的执行记录
        :param result_dict: 接口执行的结果
        :param user_info: 用户信息
        :return: None,err
        """
        try:
            if user_info:
                self.user_info, is_pass = force_transform_type(variable=user_info, var_type="only_dict", default={})
            user_id = self.user_info.get("user_id", None)
            if not user_id:
                return None, "不存在用户信息"
            result_dict, is_pass = force_transform_type(variable=result_dict, var_type="only_dict", default={})
            result_dict.update(self.source_params)
            result_dict.update(self.user_info)
            # 记录字段
            flow_node_id = self.flow_node_dict.get("node_id")
            flow_action_id = self.action_info_dict.get("action_id")
            relate_data_key = self.flow_action_node_map.get("relate_data_key")
            relate_data_value = ""
            if isinstance(result_dict, dict):
                relate_data_value = result_dict.get(relate_data_key, None) if relate_data_key else ""

            # 判断是否保存
            if user_id and flow_node_id and flow_action_id:
                FlowRecord.objects.create(
                    user_id=user_id,
                    flow_node_id=flow_node_id,
                    flow_action_id=flow_action_id,
                    relate_data_key=relate_data_key,
                    relate_data_value=relate_data_value,
                    request_params=self.source_params,
                    process_context=self.process_context
                )
                return None, None
            return None, "并没有有效的执行流程"
        except Exception as e:
            return None, str(e)



from django.db.models import F

from ..models import Flow, FlowNode, FlowAction, FlowNodeToAction, FlowNodeActionRule, FlowActionToOperator
from ..utils.custom_tool import format_params_handle, filter_result_field


class FlowBasicService:

    @staticmethod
    def get_flow_list(category_id=None, flow_name=None):
        """
        获取流程列表
        @param category_id 类别ID
        @param flow_name 流程名称
        """

        flow_set = Flow.objects.all()
        if category_id:
            flow_set = flow_set.filter(category_id=category_id)
        if flow_name:
            flow_set = flow_set.filter(flow_name__contains=flow_name)
        # print("flow_set:", flow_set)

        return list(flow_set.values()), None

    @staticmethod
    def get_flow_node_list(params=None):
        """
        获取流程节点列表
        :param params: 请求参数
        """
        if params is None:
            params = {}
        sort = params.get("sort", "flow_number")
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["flow_id|int", "node_name", "node_value", "status_code"],
            is_remove_empty=True
        )
        flow_node_set = FlowNode.objects.order_by(sort).filter(**params)
        flow_node_values = flow_node_set.values()
        return list(flow_node_values), None

    @staticmethod
    def get_flow_action_list(search_params=None):
        """
        获取流程动作列表
        :param search_params: 搜索参数
        """
        # 如果不传绑定参数则表示返回所有的动作列表
        if search_params is None:
            search_params = {}
        user_id = search_params.get("user_id", None)
        role_id = search_params.get("role_id", None)
        flow_node_id = search_params.get("flow_node_id", None)

        if user_id or role_id:
            operator_set = FlowActionToOperator.objects.all()
            if user_id:
                operator_set = operator_set.filter(user_id=user_id)
            if role_id:
                operator_set = operator_set.filter(role_id=role_id)
            operator_action_id_list = [it.flow_action_id for it in operator_set]
            # 如果没有找到操作人，就不应该也不能再往下匹配了，直接返回空列表
            if not operator_action_id_list:
                return [], None

            action_list = list(FlowAction.objects.filter(id__in=operator_action_id_list).values())
            return action_list, None
        elif flow_node_id:
            action_set = FlowNodeToAction.objects.filter(flow_node_id=flow_node_id).annotate(
                action=F("flow_action_id__action"),
                action_id=F("flow_action_id__id"),
                name=F("flow_action_id__name"),
                action_description=F("flow_action_id__description"),
                config=F("flow_action_id__config"),
                service_config=F("flow_action_id__service_config")
            ).values("action", "action_id", "name", "action_description", "config", "service_config")
            action_list = filter_result_field(
                result_list=list(action_set),
                alias_dict={"action_id": "id", "action_description": "description"}
            )
            return action_list, None
        else:
            search_params = format_params_handle(
                param_dict=search_params,
                filter_filed_list=["id", "action_id", "action", "name"],
                alias_dict={"action_id": "id"},
                is_remove_empty=True
            )
            action_set = FlowAction.objects.filter(**search_params)
            action_list = list(action_set.values())
            return action_list, None

    @staticmethod
    def get_flow_node_to_action_list(params=None, need_custom_serialize=False):
        """
        获取流程动作列表
        :param params: 搜索参数
        :param need_custom_serialize: 是否使用自定义的序列化工具，进行序列化数据。
        :return data,err
        """
        # need_custom_serialize = True
        if params is None:
            params = {}
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "flow_id|int", "flow_node_id|int", "flow_action_id|int", "is_using|int", "flow_node_value", "flow_action", 'flow_status_code_list|list_int'
            ],
            split_list=["flow_status_code_list"],
            alias_dict={"flow_status_code_list": "flow_status_code__in", "flow_action": "flow_action_value"}
        )
        result_set = FlowNodeToAction.objects.annotate(
            flow_id=F("flow_node__flow_id"),
            flow_node_value=F('flow_to_node__node_value'),
            flow_node_name=F('flow_node__node_name'),
            flow_node_config=F('flow_node__config'),
            flow_status_code=F('flow_node__status_code'),
            flow_action_value=F('flow_action__action'),
            flow_action_name=F('flow_action__name'),
            flow_action_config=F('flow_action__config'),
            flow_to_node_name=F('flow_to_node__node_name'),
        ).filter(**params)
        result_list = filter_result_field(
            result_list=list(
                result_set.values(
                    "id", "flow_id", "flow_node_id", "flow_node_name", "flow_node_value", "flow_node_config",
                    "flow_status_code", "flow_action_value", "flow_action_id", "flow_action_name",
                    "flow_action_config", "flow_to_node_id", "flow_to_node_name", "is_using", "api", "relate_data_key"
                )
            ),
            alias_dict={"flow_action_value": "flow_action"}
        )
        if need_custom_serialize:
            return FlowBasicService.custom_node_to_action_serialize(result_list)
        return result_list, None

    @staticmethod
    def custom_node_to_action_serialize(node_to_action_list=None):
        """
        对获取流程动作列表重新组合
        :param node_to_action_list get_flow_node_to_action_list的结果集
        :return: code_map
        """
        try:
            if node_to_action_list is None:
                node_to_action_list = []

            code_map = {}
            for i in node_to_action_list:
                nodel_info = {
                    "flow_node_id": i["flow_node_id"],
                    "flow_node_name": i["flow_node_name"],
                    "flow_node_value": i["flow_node_value"],
                    "flow_node_config": i["flow_node_config"],
                }
                action_info = {
                    "flow_action": i["flow_action"],
                    "flow_action_id": i["flow_action_id"],
                    "flow_action_name": i["flow_action_name"],
                    "flow_action_config": i["flow_action_config"]
                }

                if code_map.get(i["flow_status_code"]):
                    code_map[i["flow_status_code"]]["action_list"].append(action_info)
                else:
                    code_map[i["flow_status_code"]] = {
                        "node_info": nodel_info,
                        "action_list": [action_info]
                    }
            return code_map, None
        except Exception as e:
            return node_to_action_list, str(e)

    @staticmethod
    def get_flow_node_action_rule_list(flow_id=None, flow_node_id=None):
        """
        获取流程节点动作规则列表
        @param flow_id 流程ID
        @param flow_node_id 流程规则ID
        """
        result_set = FlowNodeActionRule.objects.all()
        result_set = result_set.annotate(
            flow_id=F('flow_node_to_action_id__flow_node_id__flow_id'),
            flow_node_id=F('flow_node_to_action_id__flow_node_id'),
            flow_action_id=F('flow_node_to_action_id__flow_action_id')
        ).filter(is_using=1)

        # 条件搜索
        if flow_id:
            result_set = result_set.filter(flow_id=flow_id)
        if flow_node_id:
            result_set = result_set.filter(flow_node_id=flow_node_id)

        result_list = list(result_set.values(
            "id",
            "flow_id",
            "flow_node_id",
            "flow_action_id",
            "rule_name",
            "rule_sort",
            "inflow_service",
            "inflow_module",
            "inflow_field",
            "outflow_module",
            "outflow_field",
            "default_value",
            "expression_string",
            "python_script",
            "is_using"
        ))

        return result_list, None



# _*_coding:utf-8_*_
from django.conf import settings
from django.conf.urls import static
from django.conf.urls import url
from django.urls import re_path

from .apis.flow_list import FlowList
from .apis.flow_node_list import FlowNodeList
from .apis.flow_node_rule_list import FlowNodeActionRuleList
from .apis.flow_process import FlowProcess
from .apis.flow_action_list import FlowActionList
from .apis.flow_node_to_action_list import FlowNodeToActionList

# 应用名称
app_name = 'xj_flow'


urlpatterns = [
    url(r'^list/?$', FlowList.as_view(), name='flow_list'),
    url(r'^node_list/?$', FlowNodeList.as_view(), name='flow_node_list'),
    url(r'^action_list/?$', FlowActionList.as_view(), name='flow_action_list'),
    url(r'^node_to_action_list/?$', FlowNodeToActionList.as_view(), name='flow_node_to_action_list'),
    url(r'^node_action_rule_list/?$', FlowNodeActionRuleList.as_view(), name='flow_node_action_rule_list'),
    url(r'^process/?$', FlowProcess.as_view(), name='flow_process'),

]



# -*- encoding:utf-8 -*-
import os


class FileOperate:

    # 如果没有目录，则创建目录
    @staticmethod
    def make_dir(folder_name):
        if not os.path.exists(folder_name):
            os.makedirs(folder_name)
        return True










# _*_coding:utf-8_*_

from rest_framework.views import APIView

from xj_user.utils.user_wrapper import user_authentication_force_wrapper
from ..services.flow_process_service import FlowProcessService
from ..utils.custom_response import util_response
from ..utils.request_params_wrapper import request_params_wrapper


class FlowProcess(APIView):

    def __init__(self, *args, **kwargs):
        self.flow_process_service = FlowProcessService()
        super().__init__(*args, **kwargs)

    @request_params_wrapper
    @user_authentication_force_wrapper
    def post(self, request, request_params=None, *args, user_info=None, **kwargs):
        """
        流程作业
        """
        flow_node_id = request_params.pop("flow_node_id", None)
        flow_action_id = request_params.pop("flow_action_id", None)
        flow_node_value = request_params.pop("flow_node_value", None)
        flow_action_value = request_params.pop("flow_action_value", None)
        if (not flow_node_id and not flow_node_value) or (not flow_action_id and not flow_action_value):
            return util_response(err=1001, msg='flow_node_id 必填')
        # 抵用流程
        data, error_text = self.flow_process_service.do_once_flow_in_service(
            flow_node_id=flow_node_id,
            flow_action_id=flow_action_id,
            flow_node_value=flow_node_value,
            flow_action_value=flow_action_value,
            source_params=request_params
        )

        # 保存记录,执行请求返回结果解析出来
        run_flow_record_params = {

        }

        # 记录流程的操作
        record_data, record_err = self.flow_process_service.save_record(result_dict=run_flow_record_params, user_info=user_info)
        if record_err:
            return util_response(err=1000, msg=error_text)
        if error_text:
            return util_response(err=1001, msg=error_text)
        return util_response(data=data)
